@@ -1,10 +1,11 @@
-/* $OpenBSD: progs.h,v 1.8 2016/09/05 10:45:19 deraadt Exp $ */
+/* $OpenBSD: progs.h,v 1.11 2024/07/08 05:59:10 tb Exp $ */
 /* Public domain */
 
 int asn1parse_main(int argc, char **argv);
 int ca_main(int argc, char **argv);
 int certhash_main(int argc, char **argv);
 int ciphers_main(int argc, char **argv);
+int cms_main(int argc, char **argv);
 int crl2pkcs7_main(int argc, char **argv);
 int crl_main(int argc, char **argv);
 int dgst_main(int argc, char **argv);
@@ -20,7 +21,6 @@ int gendh_main(int argc, char **argv);
 int gendsa_main(int argc, char **argv);
 int genpkey_main(int argc, char **argv);
 int genrsa_main(int argc, char **argv);
-int nseq_main(int argc, char **argv);
 int ocsp_main(int argc, char **argv);
 int passwd_main(int argc, char **argv);
 int pkcs7_main(int argc, char **argv);
@@ -40,7 +40,6 @@ int s_time_main(int argc, char **argv);
 int sess_id_main(int argc, char **argv);
 int smime_main(int argc, char **argv);
 int speed_main(int argc, char **argv);
-int spkac_main(int argc, char **argv);
 int ts_main(int argc, char **argv);
 int verify_main(int argc, char **argv);
 int version_main(int argc, char **argv);
