@@ -1,4 +1,4 @@
-/* $OpenBSD: cms_lib.c,v 1.16 2022/11/26 16:08:51 tb Exp $ */
+/* $OpenBSD: cms_lib.c,v 1.25 2024/03/30 01:53:05 joshua Exp $ */
 /*
  * Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project.
@@ -69,24 +69,28 @@ d2i_CMS_ContentInfo(CMS_ContentInfo **a, const unsigned char **in, long len)
 	return (CMS_ContentInfo *)ASN1_item_d2i((ASN1_VALUE **)a, in, len,
 	    &CMS_ContentInfo_it);
 }
+LCRYPTO_ALIAS(d2i_CMS_ContentInfo);
 
 int
 i2d_CMS_ContentInfo(CMS_ContentInfo *a, unsigned char **out)
 {
 	return ASN1_item_i2d((ASN1_VALUE *)a, out, &CMS_ContentInfo_it);
 }
+LCRYPTO_ALIAS(i2d_CMS_ContentInfo);
 
 CMS_ContentInfo *
 CMS_ContentInfo_new(void)
 {
 	return (CMS_ContentInfo *)ASN1_item_new(&CMS_ContentInfo_it);
 }
+LCRYPTO_ALIAS(CMS_ContentInfo_new);
 
 void
 CMS_ContentInfo_free(CMS_ContentInfo *a)
 {
 	ASN1_item_free((ASN1_VALUE *)a, &CMS_ContentInfo_it);
 }
+LCRYPTO_ALIAS(CMS_ContentInfo_free);
 
 int
 CMS_ContentInfo_print_ctx(BIO *out, CMS_ContentInfo *x, int indent, const ASN1_PCTX *pctx)
@@ -94,12 +98,14 @@ CMS_ContentInfo_print_ctx(BIO *out, CMS_ContentInfo *x, int indent, const ASN1_P
 	return ASN1_item_print(out, (ASN1_VALUE *)x, indent,
 	    &CMS_ContentInfo_it, pctx);
 }
+LCRYPTO_ALIAS(CMS_ContentInfo_print_ctx);
 
 const ASN1_OBJECT *
 CMS_get0_type(const CMS_ContentInfo *cms)
 {
 	return cms->contentType;
 }
+LCRYPTO_ALIAS(CMS_get0_type);
 
 CMS_ContentInfo *
 cms_Data_create(void)
@@ -115,78 +121,71 @@ cms_Data_create(void)
 	return cms;
 }
 
-BIO *
+static BIO *
 cms_content_bio(CMS_ContentInfo *cms)
 {
-	ASN1_OCTET_STRING **pos = CMS_get0_content(cms);
+	ASN1_OCTET_STRING **pos;
 
-	if (!pos)
+	if ((pos = CMS_get0_content(cms)) == NULL)
 		return NULL;
-	/* If content detached data goes nowhere: create NULL BIO */
-	if (!*pos)
+
+	/* If content is detached, data goes nowhere: create null BIO. */
+	if (*pos == NULL)
 		return BIO_new(BIO_s_null());
-	/*
-	 * If content not detached and created return memory BIO
-	 */
-	if (!*pos || ((*pos)->flags == ASN1_STRING_FLAG_CONT))
+
+	/* If content is not detached and was created, return memory BIO. */
+	if ((*pos)->flags == ASN1_STRING_FLAG_CONT)
 		return BIO_new(BIO_s_mem());
 
-	/* Else content was read in: return read only BIO for it */
+	/* Else content was read in: return read-only BIO for it. */
 	return BIO_new_mem_buf((*pos)->data, (*pos)->length);
 }
 
 BIO *
-CMS_dataInit(CMS_ContentInfo *cms, BIO *icont)
+CMS_dataInit(CMS_ContentInfo *cms, BIO *in_content_bio)
 {
-	BIO *cmsbio, *cont;
+	BIO *cms_bio = NULL, *content_bio = NULL;
 
-	if (icont)
-		cont = icont;
-	else
-		cont = cms_content_bio(cms);
-	if (!cont) {
+	if ((content_bio = in_content_bio) == NULL)
+		content_bio = cms_content_bio(cms);
+	if (content_bio == NULL) {
 		CMSerror(CMS_R_NO_CONTENT);
-		return NULL;
+		goto err;
 	}
+
 	switch (OBJ_obj2nid(cms->contentType)) {
-
 	case NID_pkcs7_data:
-		return cont;
-
+		return content_bio;
 	case NID_pkcs7_signed:
-		cmsbio = cms_SignedData_init_bio(cms);
+		if ((cms_bio = cms_SignedData_init_bio(cms)) == NULL)
+			goto err;
 		break;
-
 	case NID_pkcs7_digest:
-		cmsbio = cms_DigestedData_init_bio(cms);
+		if ((cms_bio = cms_DigestedData_init_bio(cms)) == NULL)
+			goto err;
 		break;
-#ifdef ZLIB
-	case NID_id_smime_ct_compressedData:
-		cmsbio = cms_CompressedData_init_bio(cms);
-		break;
-#endif
-
 	case NID_pkcs7_encrypted:
-		cmsbio = cms_EncryptedData_init_bio(cms);
+		if ((cms_bio = cms_EncryptedData_init_bio(cms)) == NULL)
+			goto err;
 		break;
-
 	case NID_pkcs7_enveloped:
-		cmsbio = cms_EnvelopedData_init_bio(cms);
+		if ((cms_bio = cms_EnvelopedData_init_bio(cms)) == NULL)
+			goto err;
 		break;
-
 	default:
 		CMSerror(CMS_R_UNSUPPORTED_TYPE);
-		return NULL;
+		goto err;
 	}
 
-	if (cmsbio)
-		return BIO_push(cmsbio, cont);
+	return BIO_push(cms_bio, content_bio);
 
-	if (!icont)
-		BIO_free(cont);
+ err:
+	if (content_bio != in_content_bio)
+		BIO_free(content_bio);
 
 	return NULL;
 }
+LCRYPTO_ALIAS(CMS_dataInit);
 
 int
 CMS_dataFinal(CMS_ContentInfo *cms, BIO *cmsbio)
@@ -233,6 +232,50 @@ CMS_dataFinal(CMS_ContentInfo *cms, BIO *cmsbio)
 		return 0;
 	}
 }
+LCRYPTO_ALIAS(CMS_dataFinal);
+
+int
+CMS_get_version(const CMS_ContentInfo *cms, long *version)
+{
+	switch (OBJ_obj2nid(cms->contentType)) {
+	case NID_pkcs7_signed:
+		*version = cms->d.signedData->version;
+		return 1;
+
+	case NID_pkcs7_enveloped:
+		*version = cms->d.envelopedData->version;
+		return 1;
+
+	case NID_pkcs7_digest:
+		*version = cms->d.digestedData->version;
+		return 1;
+
+	case NID_pkcs7_encrypted:
+		*version = cms->d.encryptedData->version;
+		return 1;
+
+	case NID_id_smime_ct_authData:
+		*version = cms->d.authenticatedData->version;
+		return 1;
+
+	case NID_id_smime_ct_compressedData:
+		*version = cms->d.compressedData->version;
+		return 1;
+
+	default:
+		CMSerror(CMS_R_UNSUPPORTED_TYPE);
+		return 0;
+	}
+}
+LCRYPTO_ALIAS(CMS_get_version);
+
+int
+CMS_SignerInfo_get_version(const CMS_SignerInfo *si, long *version)
+{
+	*version = si->version;
+	return 1;
+}
+LCRYPTO_ALIAS(CMS_SignerInfo_get_version);
 
 /*
  * Return an OCTET STRING pointer to content. This allows it to be accessed
@@ -271,6 +314,7 @@ CMS_get0_content(CMS_ContentInfo *cms)
 		return NULL;
 	}
 }
+LCRYPTO_ALIAS(CMS_get0_content);
 
 /*
  * Return an ASN1_OBJECT pointer to content type. This allows it to be
@@ -316,6 +360,7 @@ CMS_get0_eContentType(CMS_ContentInfo *cms)
 
 	return NULL;
 }
+LCRYPTO_ALIAS(CMS_get0_eContentType);
 
 int
 CMS_set1_eContentType(CMS_ContentInfo *cms, const ASN1_OBJECT *oid)
@@ -335,6 +380,7 @@ CMS_set1_eContentType(CMS_ContentInfo *cms, const ASN1_OBJECT *oid)
 
 	return 1;
 }
+LCRYPTO_ALIAS(CMS_set1_eContentType);
 
 int
 CMS_is_detached(CMS_ContentInfo *cms)
@@ -349,6 +395,7 @@ CMS_is_detached(CMS_ContentInfo *cms)
 
 	return 1;
 }
+LCRYPTO_ALIAS(CMS_is_detached);
 
 int
 CMS_set_detached(CMS_ContentInfo *cms, int detached)
@@ -376,6 +423,7 @@ CMS_set_detached(CMS_ContentInfo *cms, int detached)
 
 	return 0;
 }
+LCRYPTO_ALIAS(CMS_set_detached);
 
 /* Create a digest BIO from an X509_ALGOR structure */
 
@@ -476,6 +524,7 @@ CMS_add0_CertificateChoices(CMS_ContentInfo *cms)
 
 	return cch;
 }
+LCRYPTO_ALIAS(CMS_add0_CertificateChoices);
 
 int
 CMS_add0_cert(CMS_ContentInfo *cms, X509 *cert)
@@ -504,6 +553,7 @@ CMS_add0_cert(CMS_ContentInfo *cms, X509 *cert)
 
 	return 1;
 }
+LCRYPTO_ALIAS(CMS_add0_cert);
 
 int
 CMS_add1_cert(CMS_ContentInfo *cms, X509 *cert)
@@ -516,6 +566,7 @@ CMS_add1_cert(CMS_ContentInfo *cms, X509 *cert)
 
 	return r;
 }
+LCRYPTO_ALIAS(CMS_add1_cert);
 
 static STACK_OF(CMS_RevocationInfoChoice) **
 cms_get0_revocation_choices(CMS_ContentInfo *cms)
@@ -558,6 +609,7 @@ CMS_add0_RevocationInfoChoice(CMS_ContentInfo *cms)
 
 	return rch;
 }
+LCRYPTO_ALIAS(CMS_add0_RevocationInfoChoice);
 
 int
 CMS_add0_crl(CMS_ContentInfo *cms, X509_CRL *crl)
@@ -572,6 +624,7 @@ CMS_add0_crl(CMS_ContentInfo *cms, X509_CRL *crl)
 
 	return 1;
 }
+LCRYPTO_ALIAS(CMS_add0_crl);
 
 int
 CMS_add1_crl(CMS_ContentInfo *cms, X509_CRL *crl)
@@ -584,6 +637,7 @@ CMS_add1_crl(CMS_ContentInfo *cms, X509_CRL *crl)
 
 	return r;
 }
+LCRYPTO_ALIAS(CMS_add1_crl);
 
 STACK_OF(X509) *
 CMS_get1_certs(CMS_ContentInfo *cms)
@@ -613,6 +667,7 @@ CMS_get1_certs(CMS_ContentInfo *cms)
 	}
 	return certs;
 }
+LCRYPTO_ALIAS(CMS_get1_certs);
 
 STACK_OF(X509_CRL) *
 CMS_get1_crls(CMS_ContentInfo *cms)
@@ -642,6 +697,7 @@ CMS_get1_crls(CMS_ContentInfo *cms)
 	}
 	return crls;
 }
+LCRYPTO_ALIAS(CMS_get1_crls);
 
 static const ASN1_OCTET_STRING *
 cms_X509_get0_subject_key_id(X509 *x)
