@@ -1,4 +1,4 @@
-/* $OpenBSD: cpt_err.c,v 1.14 2022/07/12 14:42:48 kn Exp $ */
+/* $OpenBSD: cpt_err.c,v 1.16 2024/06/24 06:43:22 tb Exp $ */
 /* ====================================================================
  * Copyright (c) 1999-2011 The OpenSSL Project.  All rights reserved.
  *
@@ -60,12 +60,14 @@
 #include <openssl/err.h>
 #include <openssl/crypto.h>
 
+#include "err_local.h"
+
 #ifndef OPENSSL_NO_ERR
 
 #define ERR_FUNC(func) ERR_PACK(ERR_LIB_CRYPTO,func,0)
 #define ERR_REASON(reason) ERR_PACK(ERR_LIB_CRYPTO,0,reason)
 
-static ERR_STRING_DATA CRYPTO_str_functs[] = {
+static const ERR_STRING_DATA CRYPTO_str_functs[] = {
 	{ERR_FUNC(CRYPTO_F_CRYPTO_GET_EX_NEW_INDEX),	"CRYPTO_get_ex_new_index"},
 	{ERR_FUNC(CRYPTO_F_CRYPTO_GET_NEW_DYNLOCKID),	"CRYPTO_get_new_dynlockid"},
 	{ERR_FUNC(CRYPTO_F_CRYPTO_GET_NEW_LOCKID),	"CRYPTO_get_new_lockid"},
@@ -79,7 +81,7 @@ static ERR_STRING_DATA CRYPTO_str_functs[] = {
 	{0, NULL}
 };
 
-static ERR_STRING_DATA CRYPTO_str_reasons[] = {
+static const ERR_STRING_DATA CRYPTO_str_reasons[] = {
 	{ERR_REASON(CRYPTO_R_FIPS_MODE_NOT_SUPPORTED), "fips mode not supported"},
 	{ERR_REASON(CRYPTO_R_NO_DYNLOCK_CREATE_CALLBACK), "no dynlock create callback"},
 	{0, NULL}
@@ -92,8 +94,9 @@ ERR_load_CRYPTO_strings(void)
 {
 #ifndef OPENSSL_NO_ERR
 	if (ERR_func_error_string(CRYPTO_str_functs[0].error) == NULL) {
-		ERR_load_strings(0, CRYPTO_str_functs);
-		ERR_load_strings(0, CRYPTO_str_reasons);
+		ERR_load_const_strings(CRYPTO_str_functs);
+		ERR_load_const_strings(CRYPTO_str_reasons);
 	}
 #endif
 }
+LCRYPTO_ALIAS(ERR_load_CRYPTO_strings);
