@@ -1,4 +1,4 @@
-/* $OpenBSD: e_aes.c,v 1.56 2024/01/04 17:38:36 tb Exp $ */
+/* $OpenBSD: e_aes.c,v 1.59 2024/09/06 09:57:32 tb Exp $ */
 /* ====================================================================
  * Copyright (c) 2001-2011 The OpenSSL Project.  All rights reserved.
  *
@@ -55,6 +55,8 @@
 
 #include <openssl/opensslconf.h>
 
+#include "crypto_internal.h"
+
 #ifndef OPENSSL_NO_AES
 #include <openssl/aes.h>
 #include <openssl/err.h>
@@ -154,7 +156,7 @@ void AES_xts_decrypt(const char *inp, char *out, size_t len,
 #include "x86_arch.h"
 
 #ifdef VPAES_ASM
-#define VPAES_CAPABLE	(OPENSSL_cpu_caps() & CPUCAP_MASK_SSSE3)
+#define VPAES_CAPABLE	(crypto_cpu_caps_ia32() & CPUCAP_MASK_SSSE3)
 #endif
 #ifdef BSAES_ASM
 #define BSAES_CAPABLE	VPAES_CAPABLE
@@ -162,7 +164,7 @@ void AES_xts_decrypt(const char *inp, char *out, size_t len,
 /*
  * AES-NI section
  */
-#define	AESNI_CAPABLE	(OPENSSL_cpu_caps() & CPUCAP_MASK_AESNI)
+#define	AESNI_CAPABLE	(crypto_cpu_caps_ia32() & CPUCAP_MASK_AESNI)
 
 int aesni_set_encrypt_key(const unsigned char *userKey, int bits,
     AES_KEY *key);
@@ -572,6 +574,7 @@ EVP_aes_128_cbc(void)
 	return &aes_128_cbc;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_cbc);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_128_ecb = {
@@ -606,6 +609,7 @@ EVP_aes_128_ecb(void)
 	return &aes_128_ecb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_ecb);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_128_ofb = {
@@ -640,6 +644,7 @@ EVP_aes_128_ofb(void)
 	return &aes_128_ofb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_ofb);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_128_cfb = {
@@ -666,7 +671,7 @@ static const EVP_CIPHER aes_128_cfb = {
 };
 
 const EVP_CIPHER *
-EVP_aes_128_cfb(void)
+EVP_aes_128_cfb128(void)
 {
 #ifdef AESNI_CAPABLE
 	return AESNI_CAPABLE ? &aesni_128_cfb : &aes_128_cfb;
@@ -674,6 +679,7 @@ EVP_aes_128_cfb(void)
 	return &aes_128_cfb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_cfb128);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_128_cfb1 = {
@@ -708,6 +714,7 @@ EVP_aes_128_cfb1(void)
 	return &aes_128_cfb1;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_cfb1);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_128_cfb8 = {
@@ -742,6 +749,7 @@ EVP_aes_128_cfb8(void)
 	return &aes_128_cfb8;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_cfb8);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_128_ctr = {
@@ -776,6 +784,7 @@ EVP_aes_128_ctr(void)
 	return &aes_128_ctr;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_ctr);
 
 
 #ifdef AESNI_CAPABLE
@@ -811,6 +820,7 @@ EVP_aes_192_cbc(void)
 	return &aes_192_cbc;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_cbc);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_ecb = {
@@ -845,6 +855,7 @@ EVP_aes_192_ecb(void)
 	return &aes_192_ecb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_ecb);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_ofb = {
@@ -879,6 +890,7 @@ EVP_aes_192_ofb(void)
 	return &aes_192_ofb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_ofb);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_cfb = {
@@ -905,7 +917,7 @@ static const EVP_CIPHER aes_192_cfb = {
 };
 
 const EVP_CIPHER *
-EVP_aes_192_cfb(void)
+EVP_aes_192_cfb128(void)
 {
 #ifdef AESNI_CAPABLE
 	return AESNI_CAPABLE ? &aesni_192_cfb : &aes_192_cfb;
@@ -913,6 +925,7 @@ EVP_aes_192_cfb(void)
 	return &aes_192_cfb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_cfb128);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_cfb1 = {
@@ -947,6 +960,7 @@ EVP_aes_192_cfb1(void)
 	return &aes_192_cfb1;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_cfb1);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_cfb8 = {
@@ -981,6 +995,7 @@ EVP_aes_192_cfb8(void)
 	return &aes_192_cfb8;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_cfb8);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_ctr = {
@@ -1015,6 +1030,7 @@ EVP_aes_192_ctr(void)
 	return &aes_192_ctr;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_ctr);
 
 
 #ifdef AESNI_CAPABLE
@@ -1050,6 +1066,7 @@ EVP_aes_256_cbc(void)
 	return &aes_256_cbc;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_cbc);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_ecb = {
@@ -1084,6 +1101,7 @@ EVP_aes_256_ecb(void)
 	return &aes_256_ecb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_ecb);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_ofb = {
@@ -1118,6 +1136,7 @@ EVP_aes_256_ofb(void)
 	return &aes_256_ofb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_ofb);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_cfb = {
@@ -1144,7 +1163,7 @@ static const EVP_CIPHER aes_256_cfb = {
 };
 
 const EVP_CIPHER *
-EVP_aes_256_cfb(void)
+EVP_aes_256_cfb128(void)
 {
 #ifdef AESNI_CAPABLE
 	return AESNI_CAPABLE ? &aesni_256_cfb : &aes_256_cfb;
@@ -1152,6 +1171,7 @@ EVP_aes_256_cfb(void)
 	return &aes_256_cfb;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_cfb128);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_cfb1 = {
@@ -1186,6 +1206,7 @@ EVP_aes_256_cfb1(void)
 	return &aes_256_cfb1;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_cfb1);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_cfb8 = {
@@ -1220,6 +1241,7 @@ EVP_aes_256_cfb8(void)
 	return &aes_256_cfb8;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_cfb8);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_ctr = {
@@ -1254,6 +1276,7 @@ EVP_aes_256_ctr(void)
 	return &aes_256_ctr;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_ctr);
 
 static int
 aes_gcm_cleanup(EVP_CIPHER_CTX *c)
@@ -1677,6 +1700,7 @@ EVP_aes_128_gcm(void)
 	return &aes_128_gcm;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_gcm);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_gcm = {
@@ -1715,6 +1739,7 @@ EVP_aes_192_gcm(void)
 	return &aes_192_gcm;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_gcm);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_gcm = {
@@ -1753,6 +1778,7 @@ EVP_aes_256_gcm(void)
 	return &aes_256_gcm;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_gcm);
 
 static int
 aes_xts_ctrl(EVP_CIPHER_CTX *c, int type, int arg, void *ptr)
@@ -1919,6 +1945,7 @@ EVP_aes_128_xts(void)
 	return &aes_128_xts;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_xts);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_xts = {
@@ -1957,6 +1984,7 @@ EVP_aes_256_xts(void)
 	return &aes_256_xts;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_xts);
 
 static int
 aes_ccm_ctrl(EVP_CIPHER_CTX *c, int type, int arg, void *ptr)
@@ -2157,6 +2185,7 @@ EVP_aes_128_ccm(void)
 	return &aes_128_ccm;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_128_ccm);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_192_ccm = {
@@ -2195,6 +2224,7 @@ EVP_aes_192_ccm(void)
 	return &aes_192_ccm;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_192_ccm);
 
 #ifdef AESNI_CAPABLE
 static const EVP_CIPHER aesni_256_ccm = {
@@ -2233,6 +2263,7 @@ EVP_aes_256_ccm(void)
 	return &aes_256_ccm;
 #endif
 }
+LCRYPTO_ALIAS(EVP_aes_256_ccm);
 
 #define EVP_AEAD_AES_GCM_TAG_LEN 16
 
@@ -2423,12 +2454,14 @@ EVP_aead_aes_128_gcm(void)
 {
 	return &aead_aes_128_gcm;
 }
+LCRYPTO_ALIAS(EVP_aead_aes_128_gcm);
 
 const EVP_AEAD *
 EVP_aead_aes_256_gcm(void)
 {
 	return &aead_aes_256_gcm;
 }
+LCRYPTO_ALIAS(EVP_aead_aes_256_gcm);
 
 typedef struct {
 	union {
@@ -2557,6 +2590,7 @@ EVP_aes_128_wrap(void)
 {
 	return &aes_128_wrap;
 }
+LCRYPTO_ALIAS(EVP_aes_128_wrap);
 
 static const EVP_CIPHER aes_192_wrap = {
 	.nid = NID_id_aes192_wrap,
@@ -2578,6 +2612,7 @@ EVP_aes_192_wrap(void)
 {
 	return &aes_192_wrap;
 }
+LCRYPTO_ALIAS(EVP_aes_192_wrap);
 
 static const EVP_CIPHER aes_256_wrap = {
 	.nid = NID_id_aes256_wrap,
@@ -2599,5 +2634,6 @@ EVP_aes_256_wrap(void)
 {
 	return &aes_256_wrap;
 }
+LCRYPTO_ALIAS(EVP_aes_256_wrap);
 
 #endif
