@@ -1,4 +1,4 @@
-/* $OpenBSD: p_lib.c,v 1.59 2024/03/02 11:17:27 tb Exp $ */
+/* $OpenBSD: p_lib.c,v 1.61 2024/08/22 12:24:24 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -170,6 +170,7 @@ EVP_PKEY_asn1_get_count(void)
 {
 	return N_ASN1_METHODS;
 }
+LCRYPTO_ALIAS(EVP_PKEY_asn1_get_count);
 
 const EVP_PKEY_ASN1_METHOD *
 EVP_PKEY_asn1_get0(int idx)
@@ -179,6 +180,7 @@ EVP_PKEY_asn1_get0(int idx)
 
 	return asn1_methods[idx];
 }
+LCRYPTO_ALIAS(EVP_PKEY_asn1_get0);
 
 const EVP_PKEY_ASN1_METHOD *
 EVP_PKEY_asn1_find(ENGINE **engine, int pkey_id)
@@ -195,6 +197,7 @@ EVP_PKEY_asn1_find(ENGINE **engine, int pkey_id)
 
 	return NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_asn1_find);
 
 const EVP_PKEY_ASN1_METHOD *
 EVP_PKEY_asn1_find_str(ENGINE **engine, const char *str, int len)
@@ -224,6 +227,7 @@ EVP_PKEY_asn1_find_str(ENGINE **engine, const char *str, int len)
 
 	return NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_asn1_find_str);
 
 int
 EVP_PKEY_asn1_get0_info(int *pkey_id, int *pkey_base_id, int *pkey_flags,
@@ -246,12 +250,14 @@ EVP_PKEY_asn1_get0_info(int *pkey_id, int *pkey_base_id, int *pkey_flags,
 
 	return 1;
 }
+LCRYPTO_ALIAS(EVP_PKEY_asn1_get0_info);
 
 const EVP_PKEY_ASN1_METHOD*
 EVP_PKEY_get0_asn1(const EVP_PKEY *pkey)
 {
 	return pkey->ameth;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0_asn1);
 
 int
 EVP_PKEY_bits(const EVP_PKEY *pkey)
@@ -260,6 +266,7 @@ EVP_PKEY_bits(const EVP_PKEY *pkey)
 		return pkey->ameth->pkey_bits(pkey);
 	return 0;
 }
+LCRYPTO_ALIAS(EVP_PKEY_bits);
 
 int
 EVP_PKEY_security_bits(const EVP_PKEY *pkey)
@@ -271,6 +278,7 @@ EVP_PKEY_security_bits(const EVP_PKEY *pkey)
 
 	return pkey->ameth->pkey_security_bits(pkey);
 }
+LCRYPTO_ALIAS(EVP_PKEY_security_bits);
 
 int
 EVP_PKEY_size(const EVP_PKEY *pkey)
@@ -279,6 +287,7 @@ EVP_PKEY_size(const EVP_PKEY *pkey)
 		return pkey->ameth->pkey_size(pkey);
 	return 0;
 }
+LCRYPTO_ALIAS(EVP_PKEY_size);
 
 int
 EVP_PKEY_save_parameters(EVP_PKEY *pkey, int mode)
@@ -303,6 +312,7 @@ EVP_PKEY_save_parameters(EVP_PKEY *pkey, int mode)
 #endif
 	return (0);
 }
+LCRYPTO_ALIAS(EVP_PKEY_save_parameters);
 
 int
 EVP_PKEY_copy_parameters(EVP_PKEY *to, const EVP_PKEY *from)
@@ -322,6 +332,7 @@ EVP_PKEY_copy_parameters(EVP_PKEY *to, const EVP_PKEY *from)
 err:
 	return 0;
 }
+LCRYPTO_ALIAS(EVP_PKEY_copy_parameters);
 
 int
 EVP_PKEY_missing_parameters(const EVP_PKEY *pkey)
@@ -330,6 +341,7 @@ EVP_PKEY_missing_parameters(const EVP_PKEY *pkey)
 		return pkey->ameth->param_missing(pkey);
 	return 0;
 }
+LCRYPTO_ALIAS(EVP_PKEY_missing_parameters);
 
 int
 EVP_PKEY_cmp_parameters(const EVP_PKEY *a, const EVP_PKEY *b)
@@ -340,6 +352,7 @@ EVP_PKEY_cmp_parameters(const EVP_PKEY *a, const EVP_PKEY *b)
 		return a->ameth->param_cmp(a, b);
 	return -2;
 }
+LCRYPTO_ALIAS(EVP_PKEY_cmp_parameters);
 
 int
 EVP_PKEY_cmp(const EVP_PKEY *a, const EVP_PKEY *b)
@@ -362,6 +375,7 @@ EVP_PKEY_cmp(const EVP_PKEY *a, const EVP_PKEY *b)
 
 	return -2;
 }
+LCRYPTO_ALIAS(EVP_PKEY_cmp);
 
 EVP_PKEY *
 EVP_PKEY_new(void)
@@ -379,12 +393,14 @@ EVP_PKEY_new(void)
 
 	return pkey;
 }
+LCRYPTO_ALIAS(EVP_PKEY_new);
 
 int
 EVP_PKEY_up_ref(EVP_PKEY *pkey)
 {
 	return CRYPTO_add(&pkey->references, 1, CRYPTO_LOCK_EVP_PKEY) > 1;
 }
+LCRYPTO_ALIAS(EVP_PKEY_up_ref);
 
 static void
 evp_pkey_free_pkey_ptr(EVP_PKEY *pkey)
@@ -406,9 +422,9 @@ EVP_PKEY_free(EVP_PKEY *pkey)
 		return;
 
 	evp_pkey_free_pkey_ptr(pkey);
-	sk_X509_ATTRIBUTE_pop_free(pkey->attributes, X509_ATTRIBUTE_free);
 	freezero(pkey, sizeof(*pkey));
 }
+LCRYPTO_ALIAS(EVP_PKEY_free);
 
 int
 EVP_PKEY_set_type(EVP_PKEY *pkey, int type)
@@ -428,6 +444,7 @@ EVP_PKEY_set_type(EVP_PKEY *pkey, int type)
 
 	return 1;
 }
+LCRYPTO_ALIAS(EVP_PKEY_set_type);
 
 int
 EVP_PKEY_set_type_str(EVP_PKEY *pkey, const char *str, int len)
@@ -447,6 +464,7 @@ EVP_PKEY_set_type_str(EVP_PKEY *pkey, const char *str, int len)
 
 	return 1;
 }
+LCRYPTO_ALIAS(EVP_PKEY_set_type_str);
 
 int
 EVP_PKEY_assign(EVP_PKEY *pkey, int type, void *key)
@@ -456,6 +474,7 @@ EVP_PKEY_assign(EVP_PKEY *pkey, int type, void *key)
 
 	return (pkey->pkey.ptr = key) != NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_assign);
 
 EVP_PKEY *
 EVP_PKEY_new_raw_private_key(int type, ENGINE *engine,
@@ -485,6 +504,7 @@ EVP_PKEY_new_raw_private_key(int type, ENGINE *engine,
 
 	return NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_new_raw_private_key);
 
 EVP_PKEY *
 EVP_PKEY_new_raw_public_key(int type, ENGINE *engine,
@@ -514,6 +534,7 @@ EVP_PKEY_new_raw_public_key(int type, ENGINE *engine,
 
 	return NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_new_raw_public_key);
 
 int
 EVP_PKEY_get_raw_private_key(const EVP_PKEY *pkey,
@@ -530,6 +551,7 @@ EVP_PKEY_get_raw_private_key(const EVP_PKEY *pkey,
 
 	return 1;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get_raw_private_key);
 
 int
 EVP_PKEY_get_raw_public_key(const EVP_PKEY *pkey,
@@ -546,6 +568,7 @@ EVP_PKEY_get_raw_public_key(const EVP_PKEY *pkey,
 
 	return 1;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get_raw_public_key);
 
 EVP_PKEY *
 EVP_PKEY_new_CMAC_key(ENGINE *e, const unsigned char *priv, size_t len,
@@ -577,12 +600,14 @@ EVP_PKEY_new_CMAC_key(ENGINE *e, const unsigned char *priv, size_t len,
 
 	return NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_new_CMAC_key);
 
 void *
 EVP_PKEY_get0(const EVP_PKEY *pkey)
 {
 	return pkey->pkey.ptr;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0);
 
 const unsigned char *
 EVP_PKEY_get0_hmac(const EVP_PKEY *pkey, size_t *len)
@@ -599,6 +624,7 @@ EVP_PKEY_get0_hmac(const EVP_PKEY *pkey, size_t *len)
 
 	return os->data;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0_hmac);
 
 #ifndef OPENSSL_NO_RSA
 RSA *
@@ -610,6 +636,7 @@ EVP_PKEY_get0_RSA(EVP_PKEY *pkey)
 	EVPerror(EVP_R_EXPECTING_AN_RSA_KEY);
 	return NULL;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0_RSA);
 
 RSA *
 EVP_PKEY_get1_RSA(EVP_PKEY *pkey)
@@ -623,6 +650,7 @@ EVP_PKEY_get1_RSA(EVP_PKEY *pkey)
 
 	return rsa;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get1_RSA);
 
 int
 EVP_PKEY_set1_RSA(EVP_PKEY *pkey, RSA *key)
@@ -632,6 +660,7 @@ EVP_PKEY_set1_RSA(EVP_PKEY *pkey, RSA *key)
 		RSA_up_ref(key);
 	return ret;
 }
+LCRYPTO_ALIAS(EVP_PKEY_set1_RSA);
 #endif
 
 #ifndef OPENSSL_NO_DSA
@@ -644,6 +673,7 @@ EVP_PKEY_get0_DSA(EVP_PKEY *pkey)
 	}
 	return pkey->pkey.dsa;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0_DSA);
 
 DSA *
 EVP_PKEY_get1_DSA(EVP_PKEY *pkey)
@@ -657,6 +687,7 @@ EVP_PKEY_get1_DSA(EVP_PKEY *pkey)
 
 	return dsa;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get1_DSA);
 
 int
 EVP_PKEY_set1_DSA(EVP_PKEY *pkey, DSA *key)
@@ -666,6 +697,7 @@ EVP_PKEY_set1_DSA(EVP_PKEY *pkey, DSA *key)
 		DSA_up_ref(key);
 	return ret;
 }
+LCRYPTO_ALIAS(EVP_PKEY_set1_DSA);
 #endif
 
 #ifndef OPENSSL_NO_EC
@@ -678,6 +710,7 @@ EVP_PKEY_get0_EC_KEY(EVP_PKEY *pkey)
 	}
 	return pkey->pkey.ec;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0_EC_KEY);
 
 EC_KEY *
 EVP_PKEY_get1_EC_KEY(EVP_PKEY *pkey)
@@ -691,6 +724,7 @@ EVP_PKEY_get1_EC_KEY(EVP_PKEY *pkey)
 
 	return key;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get1_EC_KEY);
 
 int
 EVP_PKEY_set1_EC_KEY(EVP_PKEY *pkey, EC_KEY *key)
@@ -700,6 +734,7 @@ EVP_PKEY_set1_EC_KEY(EVP_PKEY *pkey, EC_KEY *key)
 		EC_KEY_up_ref(key);
 	return ret;
 }
+LCRYPTO_ALIAS(EVP_PKEY_set1_EC_KEY);
 #endif
 
 
@@ -713,6 +748,7 @@ EVP_PKEY_get0_DH(EVP_PKEY *pkey)
 	}
 	return pkey->pkey.dh;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get0_DH);
 
 DH *
 EVP_PKEY_get1_DH(EVP_PKEY *pkey)
@@ -726,6 +762,7 @@ EVP_PKEY_get1_DH(EVP_PKEY *pkey)
 
 	return dh;
 }
+LCRYPTO_ALIAS(EVP_PKEY_get1_DH);
 
 int
 EVP_PKEY_set1_DH(EVP_PKEY *pkey, DH *key)
@@ -735,6 +772,7 @@ EVP_PKEY_set1_DH(EVP_PKEY *pkey, DH *key)
 		DH_up_ref(key);
 	return ret;
 }
+LCRYPTO_ALIAS(EVP_PKEY_set1_DH);
 #endif
 
 int
@@ -747,18 +785,21 @@ EVP_PKEY_type(int type)
 
 	return NID_undef;
 }
+LCRYPTO_ALIAS(EVP_PKEY_type);
 
 int
 EVP_PKEY_id(const EVP_PKEY *pkey)
 {
 	return pkey->type;
 }
+LCRYPTO_ALIAS(EVP_PKEY_id);
 
 int
 EVP_PKEY_base_id(const EVP_PKEY *pkey)
 {
 	return EVP_PKEY_type(pkey->type);
 }
+LCRYPTO_ALIAS(EVP_PKEY_base_id);
 
 static int
 unsup_alg(BIO *out, const EVP_PKEY *pkey, int indent, const char *kstr)
@@ -779,6 +820,7 @@ EVP_PKEY_print_public(BIO *out, const EVP_PKEY *pkey, int indent,
 
 	return unsup_alg(out, pkey, indent, "Public Key");
 }
+LCRYPTO_ALIAS(EVP_PKEY_print_public);
 
 int
 EVP_PKEY_print_private(BIO *out, const EVP_PKEY *pkey, int indent,
@@ -789,6 +831,7 @@ EVP_PKEY_print_private(BIO *out, const EVP_PKEY *pkey, int indent,
 
 	return unsup_alg(out, pkey, indent, "Private Key");
 }
+LCRYPTO_ALIAS(EVP_PKEY_print_private);
 
 int
 EVP_PKEY_print_params(BIO *out, const EVP_PKEY *pkey, int indent,
@@ -798,6 +841,7 @@ EVP_PKEY_print_params(BIO *out, const EVP_PKEY *pkey, int indent,
 		return pkey->ameth->param_print(out, pkey, indent, pctx);
 	return unsup_alg(out, pkey, indent, "Parameters");
 }
+LCRYPTO_ALIAS(EVP_PKEY_print_params);
 
 int
 EVP_PKEY_get_default_digest_nid(EVP_PKEY *pkey, int *pnid)
@@ -807,3 +851,4 @@ EVP_PKEY_get_default_digest_nid(EVP_PKEY *pkey, int *pnid)
 	return pkey->ameth->pkey_ctrl(pkey, ASN1_PKEY_CTRL_DEFAULT_MD_NID,
 	    0, pnid);
 }
+LCRYPTO_ALIAS(EVP_PKEY_get_default_digest_nid);
