@@ -1,4 +1,4 @@
-/*	$OpenBSD: hmac.h,v 1.1 2022/11/11 11:25:18 beck Exp $	*/
+/*	$OpenBSD: hmac.h,v 1.3 2024/08/31 10:42:21 tb Exp $	*/
 /*
  * Copyright (c) 2016 Philip Guenther <guenther@openbsd.org>
  *
@@ -18,17 +18,16 @@
 #ifndef _LIBCRYPTO_HMAC_H_
 #define _LIBCRYPTO_HMAC_H_
 
-#ifdef _MSC_VER
-#include <../include/openssl/hmac.h>
-#else
+#ifndef _MSC_VER
 #include_next <openssl/hmac.h>
+#else
+#include "../include/openssl/hmac.h"
 #endif
 #include "crypto_namespace.h"
 
 LCRYPTO_USED(HMAC_CTX_new);
 LCRYPTO_USED(HMAC_CTX_free);
 LCRYPTO_UNUSED(HMAC_CTX_reset);
-LCRYPTO_UNUSED(HMAC_Init);
 LCRYPTO_USED(HMAC_Init_ex);
 LCRYPTO_USED(HMAC_Update);
 LCRYPTO_USED(HMAC_Final);
