@@ -1,4 +1,4 @@
-/* $OpenBSD: p12_crt.c,v 1.20 2022/08/20 09:16:18 tb Exp $ */
+/* $OpenBSD: p12_crt.c,v 1.26 2024/08/22 12:22:42 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project.
  */
@@ -60,27 +60,15 @@
 
 #include <openssl/err.h>
 #include <openssl/pkcs12.h>
+#include <openssl/x509.h>
 
+#include "evp_local.h"
 #include "pkcs12_local.h"
+#include "x509_local.h"
 
 static int pkcs12_add_bag(STACK_OF(PKCS12_SAFEBAG) **pbags,
     PKCS12_SAFEBAG *bag);
 
-static int
-copy_bag_attr(PKCS12_SAFEBAG *bag, EVP_PKEY *pkey, int nid)
-{
-	int idx;
-	X509_ATTRIBUTE *attr;
-
-	idx = EVP_PKEY_get_attr_by_NID(pkey, nid, -1);
-	if (idx < 0)
-		return 1;
-	attr = EVP_PKEY_get_attr(pkey, idx);
-	if (!X509at_add1_attr(&bag->attrib, attr))
-		return 0;
-	return 1;
-}
-
 PKCS12 *
 PKCS12_create(const char *pass, const char *name, EVP_PKEY *pkey, X509 *cert,
     STACK_OF(X509) *ca, int nid_key, int nid_cert, int iter, int mac_iter,
@@ -113,7 +101,8 @@ PKCS12_create(const char *pass, const char *name, EVP_PKEY *pkey, X509 *cert,
 	if (pkey && cert) {
 		if (!X509_check_private_key(cert, pkey))
 			return NULL;
-		X509_digest(cert, EVP_sha1(), keyid, &keyidlen);
+		if (!X509_digest(cert, EVP_sha1(), keyid, &keyidlen))
+			return NULL;
 	}
 
 	if (cert) {
@@ -142,11 +131,6 @@ PKCS12_create(const char *pass, const char *name, EVP_PKEY *pkey, X509 *cert,
 		if (!bag)
 			goto err;
 
-		if (!copy_bag_attr(bag, pkey, NID_ms_csp_name))
-			goto err;
-		if (!copy_bag_attr(bag, pkey, NID_LocalKeySet))
-			goto err;
-
 		if (name && !PKCS12_add_friendlyname(bag, name, -1))
 			goto err;
 		if (keyidlen && !PKCS12_add_localkeyid(bag, keyid, keyidlen))
@@ -183,6 +167,7 @@ err:
 		sk_PKCS12_SAFEBAG_pop_free(bags, PKCS12_SAFEBAG_free);
 	return NULL;
 }
+LCRYPTO_ALIAS(PKCS12_create);
 
 PKCS12_SAFEBAG *
 PKCS12_add_cert(STACK_OF(PKCS12_SAFEBAG) **pbags, X509 *cert)
