@@ -1,4 +1,4 @@
-/* $OpenBSD: x509_vfy.c,v 1.142 2024/03/02 10:40:05 tb Exp $ */
+/* $OpenBSD: x509_vfy.c,v 1.145 2024/08/28 07:37:50 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -646,7 +646,7 @@ X509_verify_cert(X509_STORE_CTX *ctx)
 	x509_verify_ctx_free(vctx);
 
 	/* if we succeed we have a chain in ctx->chain */
-	return (chain_count > 0 && ctx->chain != NULL);
+	return chain_count > 0 && ctx->chain != NULL;
 }
 LCRYPTO_ALIAS(X509_verify_cert);
 
@@ -1012,7 +1012,7 @@ check_crl_time(X509_STORE_CTX *ctx, X509_CRL *crl, int notify)
 	if (ctx->param->flags & X509_V_FLAG_USE_CHECK_TIME)
 		ptime = &ctx->param->check_time;
 	else if (ctx->param->flags & X509_V_FLAG_NO_CHECK_TIME)
-		return (1);
+		return 1;
 	else
 		ptime = NULL;
 
@@ -1744,18 +1744,6 @@ verify_cb_cert(X509_STORE_CTX *ctx, X509 *x, int depth, int err)
 	return ctx->verify_cb(0, ctx);
 }
 
-
-/* Mimic OpenSSL '0 for failure' ick */
-static int
-time_t_bogocmp(time_t a, time_t b)
-{
-	if (a == -1 || b == -1)
-		return 0;
-	if (a <= b)
-		return -1;
-	return 1;
-}
-
 /*
  * Check certificate validity times.
  *
@@ -1777,10 +1765,7 @@ x509_check_cert_time(X509_STORE_CTX *ctx, X509 *x, int depth)
 	else
 		ptime = time(NULL);
 
-	if (x->ex_flags & EXFLAG_SET)
-		i = time_t_bogocmp(x->not_before, ptime);
-	else
-		i = X509_cmp_time(X509_get_notBefore(x), &ptime);
+	i = X509_cmp_time(X509_get_notBefore(x), &ptime);
 
 	if (i >= 0 && depth < 0)
 		return 0;
@@ -1791,10 +1776,7 @@ x509_check_cert_time(X509_STORE_CTX *ctx, X509 *x, int depth)
 	    X509_V_ERR_CERT_NOT_YET_VALID))
 		return 0;
 
-	if (x->ex_flags & EXFLAG_SET)
-		i = time_t_bogocmp(x->not_after, ptime);
-	else
-		i = X509_cmp_time_internal(X509_get_notAfter(x), &ptime, 1);
+	i = X509_cmp_time_internal(X509_get_notAfter(x), &ptime, 1);
 
 	if (i <= 0 && depth < 0)
 		return 0;
@@ -2316,7 +2298,7 @@ X509_STORE_CTX_init(X509_STORE_CTX *ctx, X509_STORE *store, X509 *leaf,
 	}
 
 	if (CRYPTO_new_ex_data(CRYPTO_EX_INDEX_X509_STORE_CTX, ctx,
-	    &(ctx->ex_data)) == 0) {
+	    &ctx->ex_data) == 0) {
 		X509error(ERR_R_MALLOC_FAILURE);
 		return 0;
 	}
@@ -2355,8 +2337,7 @@ X509_STORE_CTX_cleanup(X509_STORE_CTX *ctx)
 		sk_X509_pop_free(ctx->chain, X509_free);
 		ctx->chain = NULL;
 	}
-	CRYPTO_free_ex_data(CRYPTO_EX_INDEX_X509_STORE_CTX,
-	    ctx, &(ctx->ex_data));
+	CRYPTO_free_ex_data(CRYPTO_EX_INDEX_X509_STORE_CTX, ctx, &ctx->ex_data);
 	memset(&ctx->ex_data, 0, sizeof(CRYPTO_EX_DATA));
 }
 LCRYPTO_ALIAS(X509_STORE_CTX_cleanup);
@@ -2560,28 +2541,11 @@ check_key_level(X509_STORE_CTX *ctx, X509 *cert)
 static int
 check_sig_level(X509_STORE_CTX *ctx, X509 *cert)
 {
-	const EVP_MD *md;
-	int bits, nid, md_nid;
+	int bits;
 
-	if ((nid = X509_get_signature_nid(cert)) == NID_undef)
+	if (!X509_get_signature_info(cert, NULL, NULL, &bits, NULL))
 		return 0;
 
-	/*
-	 * Look up signature algorithm digest.
-	 */
-
-	if (!OBJ_find_sigid_algs(nid, &md_nid, NULL))
-		return 0;
-
-	if (md_nid == NID_undef)
-		return 0;
-
-	if ((md = EVP_get_digestbynid(md_nid)) == NULL)
-		return 0;
-
-	/* Assume 4 bits of collision resistance for each hash octet. */
-	bits = EVP_MD_size(md) * 4;
-
 	return enough_bits_for_security_level(bits, ctx->param->security_level);
 }
 
