@@ -1,4 +1,4 @@
-/*	$OpenBSD: bs_ber.c,v 1.9 2016/12/03 12:34:35 jsing Exp $	*/
+/*	$OpenBSD: bs_ber.c,v 1.12 2024/05/25 15:14:26 tb Exp $	*/
 /*
  * Copyright (c) 2014, Google Inc.
  *
@@ -12,12 +12,12 @@
  * SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. */
+ * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
 
+#include <stdint.h>
 #include <string.h>
 
-#include <openssl/opensslconf.h>
-
 #include "bytestring.h"
 
 /*
