@@ -1,8 +1,8 @@
-/* $OpenBSD: ssl_tlsext.c,v 1.137 2023/04/28 18:14:59 tb Exp $ */
+/* $OpenBSD: ssl_tlsext.c,v 1.154 2024/07/09 12:27:27 beck Exp $ */
 /*
  * Copyright (c) 2016, 2017, 2019 Joel Sing <jsing@openbsd.org>
  * Copyright (c) 2017 Doug Hogan <doug@openbsd.org>
- * Copyright (c) 2018-2019 Bob Beck <beck@openbsd.org>
+ * Copyright (c) 2018-2019, 2024 Bob Beck <beck@openbsd.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -33,6 +33,7 @@
 #include "ssl_tlsext.h"
 
 #define TLSEXT_TYPE_alpn TLSEXT_TYPE_application_layer_protocol_negotiation
+#define TLSEXT_MAX_SUPPORTED_GROUPS 64
 
 /*
  * Supported Application-Layer Protocol Negotiation - RFC 7301
@@ -86,7 +87,7 @@ tlsext_alpn_check_format(CBS *cbs)
 }
 
 static int
-tlsext_alpn_server_parse(SSL *s, uint16_t msg_types, CBS *cbs, int *alert)
+tlsext_alpn_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS alpn, selected_cbs;
 	const unsigned char *selected;
@@ -95,7 +96,6 @@ tlsext_alpn_server_parse(SSL *s, uint16_t msg_types, CBS *cbs, int *alert)
 
 	if (!CBS_get_u16_length_prefixed(cbs, &alpn))
 		return 0;
-
 	if (!tlsext_alpn_check_format(&alpn))
 		return 0;
 
@@ -109,8 +109,7 @@ tlsext_alpn_server_parse(SSL *s, uint16_t msg_types, CBS *cbs, int *alert)
 	 * 3. TLSv1.2 and earlier: ensure that SNI has already been processed.
 	 */
 	r = s->ctx->alpn_select_cb(s, &selected, &selected_len,
-	    CBS_data(&alpn), CBS_len(&alpn),
-	    s->ctx->alpn_select_cb_arg);
+	    CBS_data(&alpn), CBS_len(&alpn), s->ctx->alpn_select_cb_arg);
 
 	if (r == SSL_TLSEXT_ERR_OK) {
 		CBS_init(&selected_cbs, selected, selected_len);
@@ -162,7 +161,7 @@ tlsext_alpn_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_alpn_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_alpn_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS list, proto;
 
@@ -229,24 +228,28 @@ tlsext_supportedgroups_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_supportedgroups_server_parse(SSL *s, uint16_t msg_type, CBS *cbs,
+tlsext_supportedgroups_server_process(SSL *s, uint16_t msg_type, CBS *cbs,
     int *alert)
 {
-	CBS grouplist;
-	uint16_t *groups;
+	uint16_t *groups = NULL;
 	size_t groups_len;
-	int i;
+	CBS grouplist;
+	int i, j;
+	int ret = 0;
 
 	if (!CBS_get_u16_length_prefixed(cbs, &grouplist))
-		return 0;
+		goto err;
 
 	groups_len = CBS_len(&grouplist);
 	if (groups_len == 0 || groups_len % 2 != 0)
-		return 0;
+		goto err;
 	groups_len /= 2;
 
+	if (groups_len > TLSEXT_MAX_SUPPORTED_GROUPS)
+		goto err;
+
 	if (s->hit)
-		return 1;
+		goto done;
 
 	if (s->s3->hs.tls13.hrr) {
 		if (s->session->tlsext_supportedgroups == NULL) {
@@ -259,33 +262,49 @@ tlsext_supportedgroups_server_parse(SSL *s, uint16_t msg_type, CBS *cbs,
 		 * did not change its list of supported groups.
 		 */
 
-		return 1;
+		goto done;
 	}
 
 	if (s->session->tlsext_supportedgroups != NULL)
-		return 0; /* XXX internal error? */
+		goto err; /* XXX internal error? */
 
 	if ((groups = reallocarray(NULL, groups_len, sizeof(uint16_t))) == NULL) {
 		*alert = SSL_AD_INTERNAL_ERROR;
-		return 0;
+		goto err;
 	}
 
 	for (i = 0; i < groups_len; i++) {
-		if (!CBS_get_u16(&grouplist, &groups[i])) {
-			free(groups);
-			return 0;
+		if (!CBS_get_u16(&grouplist, &groups[i]))
+			goto err;
+		/*
+		 * Do not allow duplicate groups to be sent. This is not
+		 * currently specified in RFC 8446 or earlier, but there is no
+		 * legitimate justification for this to occur in TLS 1.2 or TLS
+		 * 1.3.
+		 */
+		for (j = 0; j < i; j++) {
+			if (groups[i] == groups[j]) {
+				*alert = SSL_AD_ILLEGAL_PARAMETER;
+				goto err;
+			}
 		}
 	}
 
-	if (CBS_len(&grouplist) != 0) {
-		free(groups);
-		return 0;
-	}
+	if (CBS_len(&grouplist) != 0)
+		goto err;
 
 	s->session->tlsext_supportedgroups = groups;
 	s->session->tlsext_supportedgroups_length = groups_len;
+	groups = NULL;
 
-	return 1;
+
+ done:
+	ret = 1;
+
+ err:
+	free(groups);
+
+	return ret;
 }
 
 /* This extension is never used by the server. */
@@ -302,18 +321,21 @@ tlsext_supportedgroups_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_supportedgroups_client_parse(SSL *s, uint16_t msg_type, CBS *cbs,
+tlsext_supportedgroups_client_process(SSL *s, uint16_t msg_type, CBS *cbs,
     int *alert)
 {
 	/*
-	 * Servers should not send this extension per the RFC.
-	 *
-	 * However, certain F5 BIG-IP systems incorrectly send it. This bug is
-	 * from at least 2014 but as of 2017, there are still large sites with
-	 * this unpatched in production. As a result, we need to currently skip
-	 * over the extension and ignore its content:
-	 *
-	 *  https://support.f5.com/csp/article/K37345003
+	 * This extension is only allowed in TLSv1.3 encrypted extensions.
+	 * It is not permitted in a ServerHello in any version of TLS.
+	 */
+	if (msg_type != SSL_TLSEXT_MSG_EE)
+		return 0;
+
+	/*
+	 * RFC 8446, section 4.2.7: TLSv1.3 servers can send this extension but
+	 * clients must not act on it during the handshake. This allows servers
+	 * to advertise their preferences for subsequent handshakes. We ignore
+	 * this complication.
 	 */
 	if (!CBS_skip(cbs, CBS_len(cbs))) {
 		*alert = SSL_AD_INTERNAL_ERROR;
@@ -351,7 +373,7 @@ tlsext_ecpf_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ecpf_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ecpf_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS ecpf;
 
@@ -391,9 +413,9 @@ tlsext_ecpf_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ecpf_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ecpf_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
-	return tlsext_ecpf_parse(s, msg_type, cbs, alert);
+	return tlsext_ecpf_process(s, msg_type, cbs, alert);
 }
 
 static int
@@ -409,9 +431,9 @@ tlsext_ecpf_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ecpf_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ecpf_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
-	return tlsext_ecpf_parse(s, msg_type, cbs, alert);
+	return tlsext_ecpf_process(s, msg_type, cbs, alert);
 }
 
 /*
@@ -440,7 +462,7 @@ tlsext_ri_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ri_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ri_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS reneg;
 
@@ -489,7 +511,7 @@ tlsext_ri_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ri_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ri_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS reneg, prev_client, prev_server;
 
@@ -572,7 +594,7 @@ tlsext_sigalgs_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_sigalgs_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_sigalgs_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS sigalgs;
 
@@ -609,7 +631,7 @@ tlsext_sigalgs_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_sigalgs_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_sigalgs_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS sigalgs;
 
@@ -736,7 +758,7 @@ tlsext_sni_is_valid_hostname(CBS *cbs, int *is_ip)
 }
 
 static int
-tlsext_sni_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_sni_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS server_name_list, host_name;
 	uint8_t name_type;
@@ -832,7 +854,7 @@ tlsext_sni_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_sni_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_sni_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	if (s->tlsext_hostname == NULL || CBS_len(cbs) != 0) {
 		*alert = SSL_AD_UNRECOGNIZED_NAME;
@@ -920,7 +942,7 @@ tlsext_ocsp_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ocsp_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ocsp_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	int alert_desc = SSL_AD_DECODE_ERROR;
 	CBS respid_list, respid, exts;
@@ -1028,7 +1050,7 @@ tlsext_ocsp_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_ocsp_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_ocsp_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	uint8_t status_type;
 	CBS response;
@@ -1148,7 +1170,7 @@ tlsext_sessionticket_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_sessionticket_server_parse(SSL *s, uint16_t msg_type, CBS *cbs,
+tlsext_sessionticket_server_process(SSL *s, uint16_t msg_type, CBS *cbs,
     int *alert)
 {
 	if (s->tls_session_ticket_ext_cb) {
@@ -1185,7 +1207,7 @@ tlsext_sessionticket_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_sessionticket_client_parse(SSL *s, uint16_t msg_type, CBS *cbs,
+tlsext_sessionticket_client_process(SSL *s, uint16_t msg_type, CBS *cbs,
     int *alert)
 {
 	if (s->tls_session_ticket_ext_cb) {
@@ -1257,7 +1279,7 @@ tlsext_srtp_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_srtp_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_srtp_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	const SRTP_PROTECTION_PROFILE *cprof, *sprof;
 	STACK_OF(SRTP_PROTECTION_PROFILE) *clnt = NULL, *srvr;
@@ -1362,7 +1384,7 @@ tlsext_srtp_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_srtp_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_srtp_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	STACK_OF(SRTP_PROTECTION_PROFILE) *clnt;
 	const SRTP_PROTECTION_PROFILE *prof;
@@ -1443,16 +1465,38 @@ tlsext_keyshare_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_keyshare_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_keyshare_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
-	CBS client_shares, key_exchange;
+	const uint16_t *client_groups = NULL, *server_groups = NULL;
+	size_t client_groups_len = 0, server_groups_len = 0;
+	size_t i, j, client_groups_index;
+	int preferred_group_found = 0;
 	int decode_error;
+	uint16_t client_preferred_group = 0;
 	uint16_t group;
+	CBS client_shares, key_exchange;
 
-	if (!CBS_get_u16_length_prefixed(cbs, &client_shares))
+	/*
+	 * RFC 8446 section 4.2.8:
+	 *
+	 * Each KeyShareEntry value MUST correspond to a group offered in the
+	 * "supported_groups" extension and MUST appear in the same order.
+	 * However, the values MAY be a non-contiguous subset of the
+	 * "supported_groups".
+	 */
+
+	if (!tlsext_extension_seen(s, TLSEXT_TYPE_supported_groups)) {
+		*alert = SSL_AD_ILLEGAL_PARAMETER;
 		return 0;
+	}
+	if (!tlsext_extension_processed(s, TLSEXT_TYPE_supported_groups)) {
+		*alert = SSL_AD_INTERNAL_ERROR;
+		return 0;
+	}
 
-	while (CBS_len(&client_shares) > 0) {
+	if (s->s3->hs.tls13.hrr) {
+		if (!CBS_get_u16_length_prefixed(cbs, &client_shares))
+			return 0;
 
 		/* Unpack client share. */
 		if (!CBS_get_u16(&client_shares, &group))
@@ -1460,22 +1504,108 @@ tlsext_keyshare_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 		if (!CBS_get_u16_length_prefixed(&client_shares, &key_exchange))
 			return 0;
 
-		/*
-		 * XXX - check key exchange against supported groups from client.
-		 * XXX - check that groups only appear once.
-		 */
+		/* There should only be one share. */
+		if (CBS_len(&client_shares) != 0)
+			return 0;
 
-		/*
-		 * Ignore this client share if we're using earlier than TLSv1.3
-		 * or we've already selected a key share.
-		 */
+		if (group != s->s3->hs.tls13.server_group) {
+			*alert = SSL_AD_ILLEGAL_PARAMETER;
+			return 0;
+		}
+
+		if (s->s3->hs.key_share != NULL) {
+			*alert = SSL_AD_INTERNAL_ERROR;
+			return 0;
+		}
+
+		/* Decode and store the selected key share. */
+		if ((s->s3->hs.key_share = tls_key_share_new(group)) == NULL) {
+			*alert = SSL_AD_INTERNAL_ERROR;
+			return 0;
+		}
+		if (!tls_key_share_peer_public(s->s3->hs.key_share,
+		    &key_exchange, &decode_error, NULL)) {
+			if (!decode_error)
+				*alert = SSL_AD_INTERNAL_ERROR;
+			return 0;
+		}
+
+		return 1;
+	}
+
+	/*
+	 * XXX similar to tls1_get_supported_group, but client pref
+	 * only - consider deduping later.
+	 */
+	/*
+	 * We are now assured of at least one client group.
+	 * Get the client and server group preference orders.
+	 */
+	tls1_get_group_list(s, 0, &server_groups, &server_groups_len);
+	tls1_get_group_list(s, 1, &client_groups, &client_groups_len);
+
+	/*
+	 * Find the group that is most preferred by the client that
+	 * we also support.
+	 */
+	for (i = 0; i < client_groups_len && !preferred_group_found; i++) {
+		if (!ssl_security_supported_group(s, client_groups[i]))
+			continue;
+		for (j = 0; j < server_groups_len; j++) {
+			if (server_groups[j] == client_groups[i]) {
+				client_preferred_group = client_groups[i];
+				preferred_group_found = 1;
+				break;
+			}
+		}
+	}
+
+	if (!CBS_get_u16_length_prefixed(cbs, &client_shares))
+		return 0;
+
+	client_groups_index = 0;
+	while (CBS_len(&client_shares) > 0) {
+		int client_sent_group;
+
+		/* Unpack client share. */
+		if (!CBS_get_u16(&client_shares, &group))
+			return 0;
+		if (!CBS_get_u16_length_prefixed(&client_shares, &key_exchange))
+			return 0;
+
+		/* Ignore this client share if we're using earlier than TLSv1.3 */
 		if (s->s3->hs.our_max_tls_version < TLS1_3_VERSION)
 			continue;
+
+		/*
+		 * Ensure the client share group was sent in supported groups,
+		 * and was sent in the same order as supported groups. The
+		 * supported groups has already been checked for duplicates.
+		 */
+		client_sent_group = 0;
+		while (client_groups_index < client_groups_len) {
+			if (group == client_groups[client_groups_index++]) {
+				client_sent_group = 1;
+				break;
+			}
+		}
+		if (!client_sent_group) {
+			*alert = SSL_AD_ILLEGAL_PARAMETER;
+			return 0;
+		}
+
+		/* Ignore this client share if we have already selected a key share */
 		if (s->s3->hs.key_share != NULL)
 			continue;
 
-		/* XXX - consider implementing server preference. */
-		if (!tls1_check_group(s, group))
+		/*
+		 * Ignore this client share if it is not for the most client
+		 * preferred supported group. This avoids a potential downgrade
+		 * situation where the client sends a client share for something
+		 * less preferred, and we choose to to use it instead of
+		 * requesting the more preferred group.
+		 */
+		if (!preferred_group_found || group != client_preferred_group)
 			continue;
 
 		/* Decode and store the selected key share. */
@@ -1530,7 +1660,7 @@ tlsext_keyshare_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_keyshare_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_keyshare_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS key_exchange;
 	int decode_error;
@@ -1605,7 +1735,7 @@ tlsext_versions_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_versions_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_versions_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS versions;
 	uint16_t version;
@@ -1652,7 +1782,7 @@ tlsext_versions_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_versions_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_versions_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	uint16_t selected_version;
 
@@ -1702,7 +1832,7 @@ tlsext_cookie_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_cookie_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_cookie_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS cookie;
 
@@ -1759,7 +1889,7 @@ tlsext_cookie_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_cookie_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_cookie_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	CBS cookie;
 
@@ -1814,7 +1944,7 @@ tlsext_psk_kex_modes_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_psk_kex_modes_server_parse(SSL *s, uint16_t msg_type, CBS *cbs,
+tlsext_psk_kex_modes_server_process(SSL *s, uint16_t msg_type, CBS *cbs,
     int *alert)
 {
 	CBS ke_modes;
@@ -1848,7 +1978,7 @@ tlsext_psk_kex_modes_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_psk_kex_modes_client_parse(SSL *s, uint16_t msg_type, CBS *cbs,
+tlsext_psk_kex_modes_client_process(SSL *s, uint16_t msg_type, CBS *cbs,
     int *alert)
 {
 	return 0;
@@ -1871,7 +2001,7 @@ tlsext_psk_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_psk_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_psk_client_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	return CBS_skip(cbs, CBS_len(cbs));
 }
@@ -1889,7 +2019,7 @@ tlsext_psk_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 }
 
 static int
-tlsext_psk_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_psk_server_process(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
 	return CBS_skip(cbs, CBS_len(cbs));
 }
@@ -1916,7 +2046,7 @@ tlsext_quic_transport_parameters_client_build(SSL *s, uint16_t msg_type,
 }
 
 static int
-tlsext_quic_transport_parameters_client_parse(SSL *s, uint16_t msg_type,
+tlsext_quic_transport_parameters_client_process(SSL *s, uint16_t msg_type,
     CBS *cbs, int *alert)
 {
 	if (!SSL_is_quic(s)) {
@@ -1951,7 +2081,7 @@ tlsext_quic_transport_parameters_server_build(SSL *s, uint16_t msg_type,
 }
 
 static int
-tlsext_quic_transport_parameters_server_parse(SSL *s, uint16_t msg_type,
+tlsext_quic_transport_parameters_server_process(SSL *s, uint16_t msg_type,
     CBS *cbs, int *alert)
 {
 	if (!SSL_is_quic(s)) {
@@ -1971,7 +2101,7 @@ tlsext_quic_transport_parameters_server_parse(SSL *s, uint16_t msg_type,
 struct tls_extension_funcs {
 	int (*needs)(SSL *s, uint16_t msg_type);
 	int (*build)(SSL *s, uint16_t msg_type, CBB *cbb);
-	int (*parse)(SSL *s, uint16_t msg_type, CBS *cbs, int *alert);
+	int (*process)(SSL *s, uint16_t msg_type, CBS *cbs, int *alert);
 };
 
 struct tls_extension {
@@ -1981,6 +2111,9 @@ struct tls_extension {
 	struct tls_extension_funcs server;
 };
 
+/*
+ * TLS extensions (in processing order).
+ */
 static const struct tls_extension tls_extensions[] = {
 	{
 		.type = TLSEXT_TYPE_supported_versions,
@@ -1989,12 +2122,26 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_versions_client_needs,
 			.build = tlsext_versions_client_build,
-			.parse = tlsext_versions_client_parse,
+			.process = tlsext_versions_client_process,
 		},
 		.server = {
 			.needs = tlsext_versions_server_needs,
 			.build = tlsext_versions_server_build,
-			.parse = tlsext_versions_server_parse,
+			.process = tlsext_versions_server_process,
+		},
+	},
+	{
+		.type = TLSEXT_TYPE_supported_groups,
+		.messages = SSL_TLSEXT_MSG_CH | SSL_TLSEXT_MSG_EE,
+		.client = {
+			.needs = tlsext_supportedgroups_client_needs,
+			.build = tlsext_supportedgroups_client_build,
+			.process = tlsext_supportedgroups_client_process,
+		},
+		.server = {
+			.needs = tlsext_supportedgroups_server_needs,
+			.build = tlsext_supportedgroups_server_build,
+			.process = tlsext_supportedgroups_server_process,
 		},
 	},
 	{
@@ -2004,12 +2151,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_keyshare_client_needs,
 			.build = tlsext_keyshare_client_build,
-			.parse = tlsext_keyshare_client_parse,
+			.process = tlsext_keyshare_client_process,
 		},
 		.server = {
 			.needs = tlsext_keyshare_server_needs,
 			.build = tlsext_keyshare_server_build,
-			.parse = tlsext_keyshare_server_parse,
+			.process = tlsext_keyshare_server_process,
 		},
 	},
 	{
@@ -2018,12 +2165,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_sni_client_needs,
 			.build = tlsext_sni_client_build,
-			.parse = tlsext_sni_client_parse,
+			.process = tlsext_sni_client_process,
 		},
 		.server = {
 			.needs = tlsext_sni_server_needs,
 			.build = tlsext_sni_server_build,
-			.parse = tlsext_sni_server_parse,
+			.process = tlsext_sni_server_process,
 		},
 	},
 	{
@@ -2032,12 +2179,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_ri_client_needs,
 			.build = tlsext_ri_client_build,
-			.parse = tlsext_ri_client_parse,
+			.process = tlsext_ri_client_process,
 		},
 		.server = {
 			.needs = tlsext_ri_server_needs,
 			.build = tlsext_ri_server_build,
-			.parse = tlsext_ri_server_parse,
+			.process = tlsext_ri_server_process,
 		},
 	},
 	{
@@ -2047,12 +2194,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_ocsp_client_needs,
 			.build = tlsext_ocsp_client_build,
-			.parse = tlsext_ocsp_client_parse,
+			.process = tlsext_ocsp_client_process,
 		},
 		.server = {
 			.needs = tlsext_ocsp_server_needs,
 			.build = tlsext_ocsp_server_build,
-			.parse = tlsext_ocsp_server_parse,
+			.process = tlsext_ocsp_server_process,
 		},
 	},
 	{
@@ -2061,26 +2208,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_ecpf_client_needs,
 			.build = tlsext_ecpf_client_build,
-			.parse = tlsext_ecpf_client_parse,
+			.process = tlsext_ecpf_client_process,
 		},
 		.server = {
 			.needs = tlsext_ecpf_server_needs,
 			.build = tlsext_ecpf_server_build,
-			.parse = tlsext_ecpf_server_parse,
-		},
-	},
-	{
-		.type = TLSEXT_TYPE_supported_groups,
-		.messages = SSL_TLSEXT_MSG_CH | SSL_TLSEXT_MSG_EE,
-		.client = {
-			.needs = tlsext_supportedgroups_client_needs,
-			.build = tlsext_supportedgroups_client_build,
-			.parse = tlsext_supportedgroups_client_parse,
-		},
-		.server = {
-			.needs = tlsext_supportedgroups_server_needs,
-			.build = tlsext_supportedgroups_server_build,
-			.parse = tlsext_supportedgroups_server_parse,
+			.process = tlsext_ecpf_server_process,
 		},
 	},
 	{
@@ -2089,12 +2222,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_sessionticket_client_needs,
 			.build = tlsext_sessionticket_client_build,
-			.parse = tlsext_sessionticket_client_parse,
+			.process = tlsext_sessionticket_client_process,
 		},
 		.server = {
 			.needs = tlsext_sessionticket_server_needs,
 			.build = tlsext_sessionticket_server_build,
-			.parse = tlsext_sessionticket_server_parse,
+			.process = tlsext_sessionticket_server_process,
 		},
 	},
 	{
@@ -2103,12 +2236,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_sigalgs_client_needs,
 			.build = tlsext_sigalgs_client_build,
-			.parse = tlsext_sigalgs_client_parse,
+			.process = tlsext_sigalgs_client_process,
 		},
 		.server = {
 			.needs = tlsext_sigalgs_server_needs,
 			.build = tlsext_sigalgs_server_build,
-			.parse = tlsext_sigalgs_server_parse,
+			.process = tlsext_sigalgs_server_process,
 		},
 	},
 	{
@@ -2117,12 +2250,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_alpn_client_needs,
 			.build = tlsext_alpn_client_build,
-			.parse = tlsext_alpn_client_parse,
+			.process = tlsext_alpn_client_process,
 		},
 		.server = {
 			.needs = tlsext_alpn_server_needs,
 			.build = tlsext_alpn_server_build,
-			.parse = tlsext_alpn_server_parse,
+			.process = tlsext_alpn_server_process,
 		},
 	},
 	{
@@ -2131,12 +2264,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_cookie_client_needs,
 			.build = tlsext_cookie_client_build,
-			.parse = tlsext_cookie_client_parse,
+			.process = tlsext_cookie_client_process,
 		},
 		.server = {
 			.needs = tlsext_cookie_server_needs,
 			.build = tlsext_cookie_server_build,
-			.parse = tlsext_cookie_server_parse,
+			.process = tlsext_cookie_server_process,
 		},
 	},
 #ifndef OPENSSL_NO_SRTP
@@ -2147,12 +2280,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_srtp_client_needs,
 			.build = tlsext_srtp_client_build,
-			.parse = tlsext_srtp_client_parse,
+			.process = tlsext_srtp_client_process,
 		},
 		.server = {
 			.needs = tlsext_srtp_server_needs,
 			.build = tlsext_srtp_server_build,
-			.parse = tlsext_srtp_server_parse,
+			.process = tlsext_srtp_server_process,
 		},
 	},
 #endif /* OPENSSL_NO_SRTP */
@@ -2162,12 +2295,12 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_quic_transport_parameters_client_needs,
 			.build = tlsext_quic_transport_parameters_client_build,
-			.parse = tlsext_quic_transport_parameters_client_parse,
+			.process = tlsext_quic_transport_parameters_client_process,
 		},
 		.server = {
 			.needs = tlsext_quic_transport_parameters_server_needs,
 			.build = tlsext_quic_transport_parameters_server_build,
-			.parse = tlsext_quic_transport_parameters_server_parse,
+			.process = tlsext_quic_transport_parameters_server_process,
 		},
 	},
 	{
@@ -2176,28 +2309,26 @@ static const struct tls_extension tls_extensions[] = {
 		.client = {
 			.needs = tlsext_psk_kex_modes_client_needs,
 			.build = tlsext_psk_kex_modes_client_build,
-			.parse = tlsext_psk_kex_modes_client_parse,
+			.process = tlsext_psk_kex_modes_client_process,
 		},
 		.server = {
 			.needs = tlsext_psk_kex_modes_server_needs,
 			.build = tlsext_psk_kex_modes_server_build,
-			.parse = tlsext_psk_kex_modes_server_parse,
+			.process = tlsext_psk_kex_modes_server_process,
 		},
 	},
 	{
-		/* MUST be last extension in CH per RFC 8446 section 4.2. */
-
 		.type = TLSEXT_TYPE_pre_shared_key,
 		.messages = SSL_TLSEXT_MSG_CH | SSL_TLSEXT_MSG_SH,
 		.client = {
 			.needs = tlsext_psk_client_needs,
 			.build = tlsext_psk_client_build,
-			.parse = tlsext_psk_client_parse,
+			.process = tlsext_psk_client_process,
 		},
 		.server = {
 			.needs = tlsext_psk_server_needs,
 			.build = tlsext_psk_server_build,
-			.parse = tlsext_psk_server_parse,
+			.process = tlsext_psk_server_process,
 		},
 	},
 };
@@ -2207,6 +2338,22 @@ static const struct tls_extension tls_extensions[] = {
 /* Ensure that extensions fit in a uint32_t bitmask. */
 CTASSERT(N_TLS_EXTENSIONS <= (sizeof(uint32_t) * 8));
 
+struct tlsext_data {
+	CBS extensions[N_TLS_EXTENSIONS];
+};
+
+static struct tlsext_data *
+tlsext_data_new(void)
+{
+	return calloc(1, sizeof(struct tlsext_data));
+}
+
+static void
+tlsext_data_free(struct tlsext_data *td)
+{
+	freezero(td, sizeof(*td));
+}
+
 uint16_t
 tls_extension_type(const struct tls_extension *extension)
 {
@@ -2220,7 +2367,8 @@ tls_extension_find(uint16_t type, size_t *tls_extensions_idx)
 
 	for (i = 0; i < N_TLS_EXTENSIONS; i++) {
 		if (tls_extensions[i].type == type) {
-			*tls_extensions_idx = i;
+			if (tls_extensions_idx != NULL)
+				*tls_extensions_idx = i;
 			return &tls_extensions[i];
 		}
 	}
@@ -2238,6 +2386,16 @@ tlsext_extension_seen(SSL *s, uint16_t type)
 	return ((s->s3->hs.extensions_seen & (1 << idx)) != 0);
 }
 
+int
+tlsext_extension_processed(SSL *s, uint16_t type)
+{
+	size_t idx;
+
+	if (tls_extension_find(type, &idx) == NULL)
+		return 0;
+	return ((s->s3->hs.extensions_processed & (1 << idx)) != 0);
+}
+
 const struct tls_extension_funcs *
 tlsext_funcs(const struct tls_extension *tlsext, int is_server)
 {
@@ -2250,7 +2408,8 @@ tlsext_funcs(const struct tls_extension *tlsext, int is_server)
 int
 tlsext_randomize_build_order(SSL *s)
 {
-	size_t idx, new_idx, psk_idx;
+	const struct tls_extension *psk_ext;
+	size_t idx, new_idx;
 	size_t alpn_idx = 0, sni_idx = 0;
 
 	free(s->tlsext_build_order);
@@ -2261,12 +2420,14 @@ tlsext_randomize_build_order(SSL *s)
 		return 0;
 	s->tlsext_build_order_len = N_TLS_EXTENSIONS;
 
-	/* RFC 8446, section 4.2: PSK must be the last extension in the CH. */
-	psk_idx = N_TLS_EXTENSIONS - 1;
-	s->tlsext_build_order[psk_idx] = &tls_extensions[psk_idx];
+	/* RFC 8446, section 4.2 - PSK MUST be the last extension in the CH. */
+	if ((psk_ext = tls_extension_find(TLSEXT_TYPE_pre_shared_key,
+	    NULL)) == NULL)
+		return 0;
+	s->tlsext_build_order[N_TLS_EXTENSIONS - 1] = psk_ext;
 
 	/* Fisher-Yates shuffle with PSK fixed. */
-	for (idx = 0; idx < psk_idx; idx++) {
+	for (idx = 0; idx < N_TLS_EXTENSIONS - 1; idx++) {
 		new_idx = arc4random_uniform(idx + 1);
 		s->tlsext_build_order[idx] = s->tlsext_build_order[new_idx];
 		s->tlsext_build_order[new_idx] = &tls_extensions[idx];
@@ -2381,7 +2542,7 @@ tlsext_clienthello_hash_extension(SSL *s, uint16_t type, CBS *cbs)
 		return 0;
 	/*
 	 * key_share data may be changed, and pre_shared_key data may
-	 * be changed
+	 * be changed.
 	 */
 	if (type == TLSEXT_TYPE_pre_shared_key || type == TLSEXT_TYPE_key_share)
 		return 1;
@@ -2392,9 +2553,9 @@ tlsext_clienthello_hash_extension(SSL *s, uint16_t type, CBS *cbs)
 }
 
 static int
-tlsext_parse(SSL *s, int is_server, uint16_t msg_type, CBS *cbs, int *alert)
+tlsext_parse(SSL *s, struct tlsext_data *td, int is_server, uint16_t msg_type,
+    CBS *cbs, int *alert)
 {
-	const struct tls_extension_funcs *ext;
 	const struct tls_extension *tlsext;
 	CBS extensions, extension_data;
 	uint16_t type;
@@ -2450,12 +2611,45 @@ tlsext_parse(SSL *s, int is_server, uint16_t msg_type, CBS *cbs, int *alert)
 			goto err;
 		s->s3->hs.extensions_seen |= (1 << idx);
 
+		CBS_dup(&extension_data, &td->extensions[idx]);
+	}
+
+	return 1;
+
+ err:
+	*alert = alert_desc;
+
+	return 0;
+}
+
+static int
+tlsext_process(SSL *s, struct tlsext_data *td, int is_server, uint16_t msg_type,
+    int *alert)
+{
+	const struct tls_extension_funcs *ext;
+	const struct tls_extension *tlsext;
+	int alert_desc;
+	size_t idx;
+
+	alert_desc = SSL_AD_DECODE_ERROR;
+
+	s->s3->hs.extensions_processed = 0;
+
+	/* Run processing for present TLS extensions, in a defined order. */
+	for (idx = 0; idx < N_TLS_EXTENSIONS; idx++) {
+		tlsext = &tls_extensions[idx];
+		if ((s->s3->hs.extensions_seen & (1 << idx)) == 0)
+			continue;
 		ext = tlsext_funcs(tlsext, is_server);
-		if (!ext->parse(s, msg_type, &extension_data, &alert_desc))
+		if (ext->process == NULL)
+			continue;
+		if (!ext->process(s, msg_type, &td->extensions[idx], &alert_desc))
 			goto err;
 
-		if (CBS_len(&extension_data) != 0)
+		if (CBS_len(&td->extensions[idx]) != 0)
 			goto err;
+
+		s->s3->hs.extensions_processed |= (1 << idx);
 	}
 
 	return 1;
@@ -2486,11 +2680,27 @@ tlsext_server_build(SSL *s, uint16_t msg_type, CBB *cbb)
 int
 tlsext_server_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
+	struct tlsext_data *td;
+	int ret = 0;
+
+	if ((td = tlsext_data_new()) == NULL)
+		goto err;
+
 	/* XXX - this should be done by the caller... */
 	if (msg_type == SSL_TLSEXT_MSG_CH)
 		tlsext_server_reset_state(s);
 
-	return tlsext_parse(s, 1, msg_type, cbs, alert);
+	if (!tlsext_parse(s, td, 1, msg_type, cbs, alert))
+		goto err;
+	if (!tlsext_process(s, td, 1, msg_type, alert))
+		goto err;
+
+	ret = 1;
+
+ err:
+	tlsext_data_free(td);
+
+	return ret;
 }
 
 static void
@@ -2511,9 +2721,25 @@ tlsext_client_build(SSL *s, uint16_t msg_type, CBB *cbb)
 int
 tlsext_client_parse(SSL *s, uint16_t msg_type, CBS *cbs, int *alert)
 {
+	struct tlsext_data *td;
+	int ret = 0;
+
+	if ((td = tlsext_data_new()) == NULL)
+		goto err;
+
 	/* XXX - this should be done by the caller... */
 	if (msg_type == SSL_TLSEXT_MSG_SH)
 		tlsext_client_reset_state(s);
 
-	return tlsext_parse(s, 0, msg_type, cbs, alert);
+	if (!tlsext_parse(s, td, 0, msg_type, cbs, alert))
+		goto err;
+	if (!tlsext_process(s, td, 0, msg_type, alert))
+		goto err;
+
+	ret = 1;
+
+ err:
+	tlsext_data_free(td);
+
+	return ret;
 }
