@@ -4,6 +4,8 @@ set -e
 quictest_bin=./quictest
 if [ -e ./quictest.exe ]; then
 	quictest_bin=./quictest.exe
+elif [ -e ./quictest.js ]; then
+	quictest_bin="node ./quictest.js"
 fi
 
 if [ -z $srcdir ]; then
